@@ -826,28 +826,6 @@ class BAMPatternSource : public CFilePatternSource {
 		ftr_t ftr;
 	};
 
-	struct orphan_mate_t {
-		orphan_mate_t() :
-			data(NULL),
-			size(0),
-			cap(0),
-			hash(0) {}
-
-		void reset() {
-			size = 0;
-			hash = 0;
-		}
-
-		bool empty() const {
-			return size == 0;
-		}
-
-		uint8_t* data;
-		uint16_t size;
-		uint16_t cap;
-		uint32_t hash;
-	};
-
 	struct BAMField {
 		enum aln_rec_field_name {
 			refID,
@@ -872,15 +850,15 @@ public:
 		const PatternParams& p) :
 		CFilePatternSource(infiles, p),
 		first_(true),
-		bam_batches_(p.nthreads),
-		bam_batch_indexes_(p.nthreads),
-		orphan_mates(p.nthreads * 2),
-		orphan_mates_mutex_(),
-		pp_(p) {
-			// uncompressed size of BGZF block is limited to 2**16 bytes
-			for (size_t i = 0; i < bam_batches_.size(); ++i) {
-				bam_batches_[i].reserve(1 << 16);
-			}
+		alignment_batch(0),
+		alignment_offset(0),
+		delta_(0),
+		pp_(p)
+		{
+			stream.zalloc = Z_NULL;
+			stream.zfree = Z_NULL;
+			stream.opaque = Z_NULL;
+			alignment_batch.reserve(1 << 16);
 		}
 
 	virtual void reset() {
@@ -894,11 +872,6 @@ public:
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const;
 
 	~BAMPatternSource() {
-		for (size_t i = 0; i < orphan_mates.size(); i++) {
-			if (orphan_mates[i].data != NULL) {
-				delete[] orphan_mates[i].data;
-			}
-		}
 	}
 
 
@@ -924,18 +897,14 @@ private:
 
 	int decompress_bgzf_block(uint8_t *dst, size_t dst_len, uint8_t *src, size_t src_len);
 	std::pair<bool, int> get_alignments(PerThreadReadBuf& pt, bool batch_a, unsigned& readi, bool lock);
-	void store_orphan_mate(const uint8_t* read, size_t read_len);
-	void get_orphaned_pairs(EList<Read>& buf_a, EList<Read>& buf_b, const size_t max_buf, unsigned& readi);
-	void get_or_store_orhaned_mate(EList<Read>& buf_a, EList<Read>& buf_b, unsigned& readi, const uint8_t *mate, size_t mate_len);
-	size_t get_matching_read(const uint8_t* rec);
 
-	static const int offset[];
+        static const int offset[];
 	static const uint8_t EOF_MARKER[];
 
-	std::vector<std::vector<uint8_t> > bam_batches_;
-	std::vector<size_t> bam_batch_indexes_;
-	std::vector<orphan_mate_t> orphan_mates;
-	MUTEX_T orphan_mates_mutex_;
+	std::vector<uint8_t> alignment_batch;
+	size_t alignment_offset;
+	size_t delta_;
+	z_stream stream;
 
 	PatternParams pp_;
 };
